//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
//
//==============================================================================
unit nmaths;
//==============================================================================
{$DEFINE CPU386}
{$N+ $A8}
//==============================================================================
interface
//==============================================================================

  procedure fastlog2_int;
  procedure fastfyl2x;
  function  fastlog2(fl2v : double) : extended;
  procedure fastfsqrt;
  function  fastroot(frv : double) : extended;

  function  twototheintpower(var tttip : integer) : extended;
  function  twototheposintpower(var tttpip : integer) : int64;

//==============================================================================
implementation
//==============================================================================

uses
  math,
  ntypes;

const
  exponent_size = 2047;
  exponent_bias = 1024;

  mantissa_size = 16384;
  mantissa_bits = 14;

  oneovermantissasize : Double = 1/mantissa_size;

var
  RootExponent        : array[0..exponent_size] of Double;
  RootMantissa        : array[0..mantissa_size+1] of Single;
  Log2Mantissa        : array[0..mantissa_size+1] of Single;

  fr_exponent         : Integer;
  fr_mantissa         : Integer;
  fl_mantissa         : Integer;

  nmaths_packer       : Array[0..3] of Integer;

//==============================================================================
  procedure fastlog2_int;
  asm
                push    eax
                fabs
                fxam
                fnstsw  ax
                test    ah,$41
                jz      @@01

                fstp    st
                fldz
                jmp     @@End

  @@01:         fxtract
                fimul   dword ptr powersoftwo[mantissa_bits*int64_size]
                fistp   fl_mantissa

                mov     eax,fl_mantissa
                sub     eax,mantissa_size
                fld     dword ptr Log2Mantissa[eax*single_size]
                fadd

  @@End:        pop     eax
  end;

  procedure fastfyl2x;
  asm
                call    fastlog2_int
                fmul
  end;

  function fastlog2(fl2v : double) : extended;
  asm
                fld     fl2v
                call    fastlog2_int
  end;

  procedure fastfsqrt;
  asm
                push    eax
                fabs
                fxam
                fnstsw  ax
                test    ah,$41
                jz      @@01

  @@00:         fstp    st
                fldz
                jmp     @@End

  @@01:         fxtract
                fimul   dword ptr powersoftwo[mantissa_bits*int64_size]
                fistp   fr_mantissa
                fistp   fr_exponent

                mov     eax,fr_mantissa
                sub     eax,mantissa_size
                fld     dword ptr RootMantissa[eax*single_size]

                mov     eax,fr_exponent
                add     eax,exponent_bias
{                jl      @@00
                cmp     eax,exponent_size
                jg      @@00}

                fmul    qword ptr RootExponent[eax*double_size]

  @@End:        pop     eax
  end;

  function fastroot(frv : double) : extended;
  asm
                fld     frv
                call    fastfsqrt
  end;

  function twototheintpower(var tttip : Integer) : Extended;
  asm
                mov      eax,dword ptr tttip
                fild     dword ptr [eax]
                fld1
                fscale
                fstp     st(1)
                fstp     @result
  end;

  function twototheposintpower(var tttpip : integer) : int64;
  asm
                mov      eax,dword ptr tttpip
                fild     dword ptr [eax]
                fld1
                fscale
                fstp     st(1)
                fistp    @result
  end;

//==============================================================================
initialization
//==============================================================================
  asm
                mov      eax,0

  @@01:         mov      fl_mantissa,eax
                fld1
                fild     fl_mantissa
                fmul     oneovermantissasize
                fadd     st,st(1)

                fld      st
                fsqrt
                fstp     dword ptr rootmantissa[eax*single_size]

                fyl2x
                fstp     dword ptr log2mantissa[eax*single_size]

                add      eax,1
                cmp      eax,mantissa_size
                jle      @@01
  end;

  for fr_exponent:=-exponent_bias to exponent_bias-1 do
    RootExponent[fr_exponent+exponent_bias]:=power(2,fr_exponent*powersoftwoinv[1]);

  nmaths_packer[0]:=0;
//==============================================================================
end.
//==============================================================================

