//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
//
//==============================================================================
unit nrand;
//==============================================================================
{$DEFINE CPU386}
{$N+ $A8 $I-}

//==============================================================================
interface
//==============================================================================
uses
  Windows,
  ntypes;
type
  value_carry_rec = packed record
  case integer of
    0: (value,carry : integer);
    1: (total : int64);
  end;

var
  dither_type       : Double = 0;

  mwc_rand_int_data : value_carry_rec;
  mwc_rand_int_prev : Integer;

  //============================================================================
  // Multiply with Carry Random Number Generator.
  //============================================================================
  function  mwc_rand_int   : Integer;
  function  sg_rand_float  : Extended;

//==============================================================================
implementation
//==============================================================================

//==============================================================================
//nrand constants, variables & types
//==============================================================================
Var
  nrand_packer  : array[0..2] of integer;

  function mwc_rand_int : Integer;
  asm
                push    edx
                mov     eax,mwc_rand_int_data.value
                mov     mwc_rand_int_prev,eax
                mov     edx,2083801278 //$7C3448BE
                mul     edx
                add     eax,mwc_rand_int_data.carry
                adc     edx,0
                mov     mwc_rand_int_data.carry,edx
                mov     mwc_rand_int_data.value,eax
                pop     edx
  end;

  function sg_rand_float : Extended;
  asm
                push    eax
                call    mwc_rand_int
                fild    mwc_rand_int_data.value
                fmul    dither_type
                fisub   mwc_rand_int_prev
                fmul    qword ptr PowersofTwoinv[32*double_size]
                pop     eax
  end;

//==============================================================================
initialization
//==============================================================================
  nrand_packer[0]:=0;
  QueryPerformanceCounter(mwc_rand_int_data.total);
  if mwc_rand_int_data.value=0 then
    mwc_rand_int_data.value:=$4F1BBCDD; // Golden Ratio x 2^31;
//==============================================================================
end.
//==============================================================================


